@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2015 - 2018, Paul Beckingham, Federico Hernandez.
+// Copyright 2016 - 2020, Thomas Lauf, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,62 +20,48 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
 #include <timew.h>
 #include <format.h>
 #include <iostream>
+#include <src/Journal.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 int CmdDelete (
   const CLI& cli,
   Rules& rules,
-  Database& database)
+  Database& database,
+  Journal& journal)
 {
+  auto verbose = rules.getBoolean ("verbose");
+
   // Gather IDs.
-  std::vector <int> ids = cli.getIds();
+  std::set <int> ids = cli.getIds ();
 
   if (ids.empty ())
-    throw std::string ("IDs must be specified. See 'timew help delete'.");
-
-  // Load the data.
-  // Note: There is no filter.
-  Interval filter;
-  auto tracked = getTracked (database, rules, filter);
-
-  bool dirty = true;
-
-  for (auto& id : ids)
   {
-    if (id > static_cast <int> (tracked.size ()))
-      throw format ("ID '@{1}' does not correspond to any tracking.", id);
+    throw std::string ("IDs must be specified. See 'timew help delete'.");
+  }
 
-    if (tracked[tracked.size() - id].synthetic && dirty)
+  journal.startTransaction ();
+
+  flattenDatabase (database, rules);
+  auto intervals = getIntervalsByIds (database, rules, ids);
+
+  for (const auto& interval : intervals)
+  {
+    database.deleteInterval (interval);
+
+    if (verbose)
     {
-      auto latest = getLatestInterval(database);
-      auto exclusions = getAllExclusions (rules, filter.range);
-
-      Interval modified {latest};
-
-      // Update database.
-      database.deleteInterval (latest);
-      for (auto& interval : flatten (modified, exclusions))
-        database.addInterval (interval);
-
-      dirty = false;
+      std::cout << "Deleted @" << interval.id << '\n';
     }
   }
 
-  // Delete intervals by id
-  for (auto& id : ids)
-  {
-    database.deleteInterval (tracked[tracked.size () - id]);
-
-    if (rules.getBoolean ("verbose"))
-      std::cout << "Deleted @" << id << '\n';
-  }
+  journal.endTransaction ();
 
   return 0;
 }
