@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2015 - 2018, Paul Beckingham, Federico Hernandez.
+// Copyright 2016 - 2020, Thomas Lauf, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,7 +20,7 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -34,9 +34,12 @@
 int CmdFill (
   const CLI& cli,
   Rules& rules,
-  Database& database)
+  Database& database,
+  Journal& journal)
 {
-  std::vector <int> ids = cli.getIds();
+  auto verbose = rules.getBoolean ("verbose");
+
+  std::set <int> ids = cli.getIds ();
 
   if (ids.empty ())
     throw std::string ("IDs must be specified. See 'timew help fill'.");
@@ -46,6 +49,8 @@ int CmdFill (
   Interval filter;
   auto tracked = getTracked (database, rules, filter);
 
+  journal.startTransaction ();
+
   // Apply tags to ids.
   for (auto& id : ids)
   {
@@ -60,11 +65,13 @@ int CmdFill (
     autoFill (rules, database, to);
     validate (cli, rules, database, to);
     std::cout << "# to " << to.dump () << "\n";
-    database.addInterval (to);
+    database.addInterval (to, verbose);
 
     // Note: Feedback generated inside autoFill().
   }
 
+  journal.endTransaction ();
+
   return 0;
 }
 
